#
# Copyright (c) 2008 Regents of the SIGNET lab, University of Padova.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University of Padova (SIGNET lab) nor the 
#    names of its contributors may be used to endorse or promote products 
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#


load libMiracle.so
load libmiraclelink.so
load libmiraclecbr.so
load libcbrtracer.so

load ../src/.libs/librtpfec.so


set ns [new Simulator]

$ns use-Miracle

proc finish {} {
    global ns tf cbr_ tfname
    puts "Application Packet Error Rate   :   [$cbr_(2) getPER]"
    puts "Application Delay               :   [$cbr_(2) getFTT]"
    puts "Tracefile                   :   $tfname"
    $ns flush-trace
    close $tf 
}       


set errorRNG [new RNG]


###########################################
#       Input parameters
###########################################

if {$argc == 3 } {

    set opt(per)  [lindex $argv 0]
    set opt(code) [lindex $argv 1]    
    set opt(run)  [lindex $argv 2]


}  else {
	puts "usage: ns $argv0  LinkPacketErrorRate  CodeType(0-2)  ReplicationNumber"
	exit
}



set rv [new RandomVariable/Uniform]
	$rv set min_ 0
	$rv set max_ 1
	$rv use-rng $errorRNG

set em [new ErrorModel]
	$em unit pkt
	$em set rate_ $opt(per)
	$em ranvar $rv

set run $opt(run)
    #puts "replication number: $run"
    for {set j 1} {$j < $run} {incr j} {
	$errorRNG next-substream
    }


set tfname "rtpfec.tr"
set tf [open $tfname  w]
$ns trace-all $tf




############# NODE 1: Sender ###############
set node_(1) [$ns create-M_Node]
set cbr_(1) [new Module/CBR]
set rtp_(1) [new Module/RTPFEC]

$rtp_(1) set codec_ $opt(code)
$cbr_(1) set period_ 0.02
$cbr_(1) set packetSize_ 1024

$node_(1) addModule 1 $rtp_(1) 0 "RTP"
$node_(1) addModule 2 $cbr_(1) 0 "APP"
$node_(1) setConnection $cbr_(1) $rtp_(1) trace
##############################################


############ Node 2: receiver###############
set node_(2) [$ns create-M_Node]
set cbr_(2) [new Module/CBR]
set rtp_(2) [new Module/RTPFEC]

$cbr_(2) set period_ 0.0176
$cbr_(2) set packetSize_ 1024

$node_(2) addModule 1 $rtp_(2) 0 "RTP"
$node_(2) addModule 2 $cbr_(2) 0 "APP"
$node_(2) setConnection $cbr_(2) $rtp_(2) trace
##############################################




#Definisco i vari link della rete

set link(1) [new Module/Link]
$link(1) delay      10m
$link(1) bandwidth  1mb
$link(1) qsize      10
$link(1) connect $node_(1) $rtp_(1) trace $node_(2) $rtp_(2) trace



$link(1) addErrorModel $em


# Scheduling transmission start & stop

$ns at 0.1   "$cbr_(1) start"
$ns at 9  "$cbr_(1) stop"
$ns at 10  "finish; $ns halt"


$ns run

