/*
 * Copyright (c) 2008 Regents of the SIGNET lab, University of Padova.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University of Padova (SIGNET lab) nor the 
 *    names of its contributors may be used to endorse or promote products 
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


//includo le librerie necessarie
#include <iostream>
#include <algorithm>
#include "rtpfec.h"
#include "rtp.h"
#include "ip.h"
#include "cbr-module.h"
#include <node-core.h>



//Definisco la classe dell'header per fare il bind con il Tcl
int hdr_rtpfec::offset_;

static class RtpFecPktClass : public PacketHeaderClass {
public:
  RtpFecPktClass() : PacketHeaderClass("PacketHeader/RTPFEC", sizeof(hdr_rtpfec)) {
    this->bind();
    bind_offset(&hdr_rtpfec::offset_);
  }
} class_rtp_fec_pkt; 




//Definisco la classe del modulo per fare il bind con la classe Tcl
static class ModuleRtpFecClass : public TclClass {
public:
	ModuleRtpFecClass() : TclClass("Module/RTPFEC") {}
	TclObject* create(int, const char*const*) {
	return (new RtpFecModule());
	}
} class_module_rtpfec;




//Specifico il Costruttore della classe
RtpFecModule::RtpFecModule()
{
	//azzero la matrice sulla quale operare l'eliminazione gaussiana
	for(int i=0;i<RIGHE;i++){
		for(int j=0;j<COLONNE;j++){
			mat[i][j]=0;
		}	
	}	
	
	//Azzero il vettore dei pacchetti spediti
	//SE NECESSARIO UTILIZZARE UNA DIVISIONE CON MODULO PER UN NUMERO MAGGIORE DI PACCHETTI
	for(int i=0 ; i<65000 ; i++){
		spediti[i]=0;
	}
	//costruttore della classe RtpFecModule

/*Faccio il bind di una variabile codec_ tcl con una variabile codec in c++.Questa determina il tipo di codifica fec
che si vuole utilizzare.Le codifiche disponibili sono le seguenti:
CODICE		TIPO CODIFICA
  0		a  b  fec(a,b)  c  d fec(c,d)  e  f  fec(e,f)
  1		a  b  c  fec(a,b,c)  d  e  f  fec(d,e,f)  g  h  i  fec(g,h,i)
  2		schema di codifica righe colonne (3*8), si inviano i primi 24 pacchetti poi i pacchetti di fec realizzati dalle colonne della matrice e 		successivamente i pacchetti di fec ottenuti dalle righe.
*/
	bind("codec_", &codec);
	
}




//Specifico il decostruttore della classe
RtpFecModule::~RtpFecModule()
{
	
	//decostruttode della classe RtpFecModule
}



//Specifico il metodo per i comandi Tcl
int  RtpFecModule::command(int argc, const char*const* argv)
{
	//codice del command
	return Module::command(argc, argv); //Serve perchè se non trova nuovi comandi passa ai comandi della classe padre
}

int m=8;
int n=3;
int pckptemp=0;
int pckp=0; 		//puntantore al vettore dei pacchetti ricevuti
int snp=1;		//valore del sn più piccolo da inviare
bool mapflag = false;	//Flag utile per il crea matrice per poter riscrivere la matrice dei coefficienti una volta superato il valore max delle righe
//Specifico il metodo che riceve i pacchetti
void RtpFecModule::recv(Packet *p)
{
	hdr_cmn *ch = HDR_CMN(p);		//creo un puntatore ch all'header COMMON del nuovo pacchetto ricevuto
	if(ch->direction() == hdr_cmn::UP){ 			//controllo se la direzione è dal basso verso l'alto
			Packet* pr = copy(p);			//Ricevo il pacchetto e ne faccio una copia
			//alla prima esecuzione riempio il vettore dei pacchetti ricevuti con lo stesso pacchetto
			while(pckptemp<RIGHE){
				pckrecv[pckp]=pr;
				pckp++;
				pckptemp++;
			}
			if(pckp==RIGHE) pckp=0;
			pckrecv[pckp]=pr;
			pckp++;
			creasnts();
			snp=findsnp();
			creamatrice();				//Creo la matrice dei coefficienti a partire dal vettore dei pacchetti ricevuti
			elgauss();				//Applico l'algoritmo di eliminazione gaussiana alla matrice dei coefficienti
			spedisci();				//creo i pacchetti e li invio al modulo superiore (cbr)
	}
	//Qui inizia la parte di codice che passa i pacchetti al layer sottostante
	else{

		if(codec==0){				//Controllo il valore del codec da utilizzare
			if(Q.size()>=1){		//Controllo se la dimensione della coda è >= 1
				Packet* pc = copy(p);
				Q.push(pc);		//Metto in coda il pacchetto
				
				hdr_cbr* cbrt1 = hdr_cbr::access(p);
				//std::cout << "Sto inviando cbr sn: " << cbrt1->sn << std::endl;
				sendDown(p);		//Invio il pacchetto
				sendpkt();		//Invoco la creazione di un nuovo pacchetto fec
			}
			else{
				Packet* pc = copy(p);
				Q.push(pc);		//Metto in coda il pacchetto
				hdr_cbr* cbrt1 = hdr_cbr::access(p);
				//std::cout << "Sto inviando cbr sn: " << cbrt1->sn << std::endl;
				sendDown(p); 		//Passo il pacchetto al modulo inferiore
			}	
		}
		else if(codec==1){
				//std::cout << "stai usando il codec: " << codec << std::endl;
				if(Q.size()>=2){		//Controllo se la dimensione della coda è >= 1
					Packet* pc = copy(p);
					Q.push(pc);		//Metto in coda il pacchetto
					sendDown(p);		//Invio il pacchetto
					sendpkt();		//Invoco la creazione di un nuovo pacchetto fec
				}
				else{
					Packet* pc = copy(p);
					Q.push(pc);		//Metto in coda il pacchetto
					sendDown(p); 		//Passo il pacchetto al modulo inferiore
				}
		}
		else if(codec==2){
				if(Q.size()>=m*n-1){		//Controllo se la dimensione della coda è >= 1
					Packet* pc = copy(p);
					Q.push(pc);		//Metto in coda il pacchetto
					sendDown(p);		//Invio il pacchetto
					sendpkt();		//Invoco la creazione di un nuovo pacchetto fec
				}
				else{
					Packet* pc = copy(p);
					Q.push(pc);		//Metto in coda il pacchetto
					sendDown(p); 		//Passo il pacchetto al modulo inferiore
				}
		}
		else{
			//se si inserisce un valore di codec non valido viene visualizzato un errere sullo standard error output
			fprintf(stderr,"Error.Codec type not supported!\n");
     			exit(1);
		}
	}

}

//Specifico il metodo che spedisce i pacchetti
void RtpFecModule::sendpkt(){
	if(codec==0){
		Packet* p = Packet::alloc(); //allocco il pacchetto da creare
		makepkt0(p);	//chiamo la funzione che genera il pacchetto
	}
	else if(codec==1){
		Packet* p = Packet::alloc();
		makepkt1(p);
	}
	else if(codec==2){
		Packet* p = Packet::alloc();
		makepkt2();
	}
	else {
	  fprintf(stderr,"Unsupported code type!\n");
	  exit(1);
	}

}




int txsn=1;	//Varibile che tiene conto dei pacchetti fec trasmessi
int destaddr;	//Variabile che memorizza l'indirizzo di destinazione
int destport;	//Variabile che memorizza la porta di destinazione
int pcktsize;	//Variabile che memorizza la dimensione del pacchetto

//Specifico il metodo che crea i pacchetti utilizzando la codifica di tipo 0
void RtpFecModule::makepkt0(Packet* p){
	double delay = 0; //definisco il delay di invio
 
	//std::cout << "stai crendo il paccetto fec" << std::endl; 
	//std::cout << "Q.size: " << Q.size() << std::endl;	
	Packet* temp1 = Q.front();	 	// Salvo su p1 il primo elemento della coda
	Packet* p1 = copy(temp1);		//Salvo una copia del pacchetto 1
	hdr_cbr* cbrh1 = hdr_cbr::access(p1);	//Accedo all'header cbr di p1
	int sn1 = cbrh1->sn;			// salvo su sn1 il suo sequence number
	//std::cout << "SN1: " << sn1 << std::endl;
	double ts1 = cbrh1->ts;			//Salvo il time stamp del pacchetto 1
	Q.pop(); 				//tolgo il primo elemento dalla coda
	//std::cout << "Q.size: " << Q.size() << std::endl;

	Packet* temp2 = Q.front();		// Salvo su p2 il secondo elemento della coda
	Packet* p2 = copy(temp2);		//creo una copia del pacchetto 2	
	hdr_cbr* cbrh2 = hdr_cbr::access(p2);	//Accedo all'header cbrdi p2
	int sn2 = cbrh2->sn;			// salvo su sn2 il suo sequence number
	//std::cout << "SN2: " << sn2 << std::endl;
	double ts2 = cbrh2->ts;			//salvo il time stamp del pacchetto 2
	Q.pop(); 				//tolgo il secondo elemento dalla coda
	//std::cout << "Q.size: " << Q.size() << std::endl;
	
	int min = minore(sn1,sn2); 		// trovo il minore tra i due sequence number
	

	//ora devo creare il pacchetto effettivo da inviare
	//carico prima l'header ip e l'header common dei pacchetti vecchi per copiarlo
	hdr_cmn* cht = hdr_cmn::access(p1);
	hdr_ip* ipt = hdr_ip::access(p1);

		
	//creo il nuovo common header
	hdr_cmn* ch = hdr_cmn::access(p);	//Creo un puntatore al nuovo common header
  	ch->uid() = txsn++;			//Setto il SN e lo incremento
  	ch->ptype() = PT_RTPFEC;		//Setto il tipo di pacchetto
	pcktsize = cht->size();			//Salvo la dimensione del pacchetto per dopo
  	ch->size() = cht->size();		//Setto la dimensione del pacchetto da inviare
  	

	//creo l'rtpfec header
	hdr_rtpfec* rfec = hdr_rtpfec::access(p);	//Creo il puntantore al nuovo header rtpfec
	rfec->snb = min;				//setto il snb come il min sn dei due pacchetti
	rfec->mask[sn1-min]=true;
	rfec->mask[sn2-min]=true;
	rfec->ts[sn1-min]=ts1;
	rfec->ts[sn2-min]=ts2;

	//std::cout << "fec di: " << sn1 << " + " << sn2 << std::endl;

	/*for(int i=0; i<=23 ; i++){
		std::cout << rfec->mask[i] << " ";   //Ciclo per stampare il contenuto della mask
	}
	std::cout << std::endl;*/


	//setto l'header ip del nuovo pacchetto
  	hdr_ip* iph = hdr_ip::access(p);		//Creo il puntatore al nuovo header ip
  	iph->daddr() = ipt->daddr();			//Setto il destination address
	//destaddr = ipt->daddr();			//Salvo il destination address
  	iph->dport() = ipt->dport();			//Setto la porta di destinazione
	//destport = ipt->dport();			//Salvo la destination port

	sendDown(p, delay);		//Invio il pacchetto al modulo sottostante
}

//Specifico il metodo che crea i pacchetti utilizzando la codifica di tipo 0
void RtpFecModule::makepkt1(Packet* p){
	double delay = 0; //definisco il delay di invio
 
	Packet* temp1 = Q.front();	 	// Salvo su p1 il primo elemento della coda
	Packet* p1 = copy(temp1);		//Salvo una copia del pacchetto 1
	hdr_cbr* cbrh1 = hdr_cbr::access(p1);	//Accedo all'header cbr di p1
	int sn1 = cbrh1->sn;			// salvo su sn1 il suo sequence number
	double ts1 = cbrh1->ts;			//Salvo il time stamp del pacchetto 1
	Q.pop(); 				//tolgo il primo elemento dalla coda

	Packet* temp2 = Q.front();		// Salvo su p2 il secondo elemento della coda
	Packet* p2 = copy(temp2);		//creo una copia del pacchetto 2	
	hdr_cbr* cbrh2 = hdr_cbr::access(p2);	//Accedo all'header cbrdi p2
	int sn2 = cbrh2->sn;			// salvo su sn2 il suo sequence number
	double ts2 = cbrh2->ts;			//salvo il time stamp del pacchetto 2
	Q.pop(); 				//tolgo il secondo elemento dalla coda

	Packet* temp3 = Q.front();		// Salvo su p3 il secondo elemento della coda
	Packet* p3 = copy(temp3);		//creo una copia del pacchetto 3	
	hdr_cbr* cbrh3 = hdr_cbr::access(p3);	//Accedo all'header cbrdi p3
	int sn3 = cbrh3->sn;			// salvo su sn3 il suo sequence number
	double ts3 = cbrh3->ts;			//salvo il time stamp del pacchetto 3
	Q.pop(); 				//tolgo il secondo elemento dalla coda

	int min = minore(sn1,sn2); 		//trovo il minore tra sn1 e sn2
	min = minore(min,sn3);			//Trovo il minore tra il min appena trovato e sn3

	//ora devo creare il pacchetto effettivo da inviare
	//carico prima l'header ip e l'header common dei pacchetti vecchi per copiarlo
	hdr_cmn* cht = hdr_cmn::access(p1);
	hdr_ip* ipt = hdr_ip::access(p1);

		
	//creo il nuovo common header
	hdr_cmn* ch = hdr_cmn::access(p);	//Creo un puntatore al nuovo common header
  	ch->uid() = txsn++;			//Setto il SN e lo incremento
  	ch->ptype() = PT_RTPFEC;		//Setto il tipo di pacchetto
	pcktsize = cht->size();			//Salvo la dimensione del pacchetto per dopo
  	ch->size() = cht->size();		//Setto la dimensione del pacchetto da inviare
  	

	//creo l'rtpfec header
	hdr_rtpfec* rfec = hdr_rtpfec::access(p);	//Creo il puntantore al nuovo header rtpfec
	rfec->snb = min;				//setto il snb come il min sn dei due pacchetti
	rfec->mask[sn1-min]=true;
	rfec->mask[sn2-min]=true;
	rfec->mask[sn3-min]=true;
	rfec->ts[sn1-min]=ts1;
	rfec->ts[sn2-min]=ts2;
	rfec->ts[sn3-min]=ts3;

	/*for(int i=0; i<=23 ; i++){
		std::cout << rfec->mask[i] << " ";   //Ciclo per stampare il contenuto della mask
	}
	std::cout << std::endl;*/

	//setto l'header ip del nuovo pacchetto
  	hdr_ip* iph = hdr_ip::access(p);		//Creo il puntatore al nuovo header ip
  	iph->daddr() = ipt->daddr();			//Setto il destination address
	//destaddr = ipt->daddr();
  	iph->dport() = ipt->dport();			//Setto la porta di destinazione
	//destport = ipt->dport();

	//Non setto l'header cbr non è necessario
  	//hdr_cbr* cbrh = HDR_CBR(p);
  	//cbrh->sn = txsn;
  	//txsn++;

	sendDown(p, delay);		//Invio il pacchetto al modulo sottostante
}

void RtpFecModule::makepkt2(){
	double delay = 0; //definisco il delay di invio
	
	Packet* tmp[m*n];
	for(int i=0 ; i<m*n ; i++){
		tmp[i]=Q.front();
		Q.pop();
	}
	
	Packet* p = Packet::alloc();
	//Carico il primo snb per questo giro
	hdr_cbr* cbrt = hdr_cbr::access(tmp[0]);
	int snbt = cbrt->sn;
	hdr_ip* ipt = hdr_ip::access(tmp[0]);
	hdr_cmn* cht = hdr_cmn::access(tmp[0]);	

	for(int i=0 ; i<n ; i++){
		hdr_cmn* ch = hdr_cmn::access(p);	//Creo un puntatore al nuovo common header
  		ch->uid() = txsn++;			//Setto il SN e lo incremento
		//std::cout << "ch uid= " << ch->uid() << std::endl;  		
		ch->ptype() = PT_RTPFEC;		//Setto il tipo di pacchetto
		pcktsize = cht->size();			//Salvo la dimensione del pacchetto per dopo
  		ch->size() = cht->size();		//Setto la dimensione del pacchetto da inviare	

		hdr_ip* iph = hdr_ip::access(p);		//Creo il puntatore al nuovo header ip
  		iph->daddr() = ipt->daddr();			//Setto il destination address
		//destaddr = ipt->daddr();
  		iph->dport() = ipt->dport();			//Setto la porta di destinazione
		//destport = ipt->dport();
	
		hdr_rtpfec* rtph = hdr_rtpfec::access(p);
		rtph->snb=snbt+m*i;
		////std::cout << "snb= " << snbt+m*i << std::endl;
		for(int j=0+m*i ; j<m*i+m ; j++){
			hdr_cbr* cbr = hdr_cbr::access(tmp[j]);
			//std::cout << "sn= " << cbr->sn << std::endl;
			rtph->mask[j-m*i]=1;
			rtph->ts[j-m*i]=cbr->ts;
		}
		sendDown(p, delay);
		p = Packet::alloc();
	}
	p = Packet::alloc();
	for(int i=0; i<m ; i++){
		hdr_cmn* ch = hdr_cmn::access(p);	//Creo un puntatore al nuovo common header
  		ch->uid() = txsn++;			//Setto il SN e lo incremento
		//std::cout << "ch uid= " << ch->uid() << std::endl;  		
		ch->ptype() = PT_RTPFEC;		//Setto il tipo di pacchetto
		pcktsize = cht->size();			//Salvo la dimensione del pacchetto per dopo
  		ch->size() = cht->size();		//Setto la dimensione del pacchetto da inviare	

		hdr_ip* iph = hdr_ip::access(p);		//Creo il puntatore al nuovo header ip
  		iph->daddr() = ipt->daddr();			//Setto il destination address
		//destaddr = ipt->daddr();
  		iph->dport() = ipt->dport();			//Setto la porta di destinazione
		//destport = ipt->dport();
	
		hdr_rtpfec* rtph = hdr_rtpfec::access(p);
		rtph->snb=snbt;
		////std::cout << "snb= " << snbt+m*i << std::endl;
		for(int j=i ; j<i+m*(n-1)+1 ; j=j+m){
			//std::cout << "j=" << j << std::endl;
			//std::cout << "j arriva a: " << i+m*(n-1)+1 << std::endl;
			hdr_cbr* cbr = hdr_cbr::access(tmp[j]);
			//std::cout << "j-i= " << j-i << std::endl;
			rtph->mask[j]=1;
			rtph->ts[j]=cbr->ts;
		}
		sendDown(p, delay);
		p = Packet::alloc();
	}
}

//metodo che trova il minore tra due parametri interi
int RtpFecModule::minore(int x,int y){
	if(x<y){return x;}
	else return y;
}

//metodo che trova il maggiore tra due parametri interi
int RtpFecModule::maggiore(int x,int y){
	if(x<y) return y;
	else return x;
}


/* Questo metodo analizza il vettore dei pacchetti ricevuti e crea una matrice contenente tutti i valori di sn con i rispettivi ts. la matrice è stata sovradimensionata in modo di poter aggiungere in futuro altre informazioni relative ai pacchetti
*/
void RtpFecModule::creasnts(){
	bool find=false;
	int sntsp=0;
	
	//Azzero la matrice snts
	for(int i=0 ; i<COLONNEMAT ; i++){
		snts[i][0]=0;
		snts[i][1]=0;
	}
	
		
	//Ciclo il vettore dei pacchetti ricevuti
	for(int i=0 ; i<RIGHE ; i++){
		Packet* tmp = pckrecv[i];    
		hdr_cmn* cht = hdr_cmn::access(tmp);
		if(cht->ptype()==PT_MCBR){   //controllo se ho un pacchetto CBR
			hdr_cbr* cbrt = hdr_cbr::access(tmp);
			hdr_ip* ipt = hdr_ip::access(tmp);
			destaddr = ipt->daddr();
			destport = ipt->dport();
			int sn=cbrt->sn;		
			//std::cout << "sn " << sn << std::endl;
			double ts=cbrt->ts;
			for(int j=0 ; j<COLONNEMAT ; j++){	//controllo se il sn attuale è già presente in snts ciclandola
				if(sn==snts[j][0]) find=true;	//se lo trovo non faccio niente		
			}
			if(!find){ //se il sn non è presente lo aggiungo in snts con il rispettivo snts
				snts[sntsp][0]=sn;	
				snts[sntsp][1]=ts;
				sntsp++;			
			}
			else find=false;
		}
		else{ //controllo se il pacchetto è FEC
			hdr_rtpfec* rtph = hdr_rtpfec::access(tmp);
			for(int j=0 ; j<MASCHERA ; j++){ //in questo modo controllo a quali pacchetti fa riferimento il pacchetto FEC
				if(rtph->mask[j]==1){
					int sn=rtph->snb+j;
					double ts=rtph->ts[j];
					for(int k=0 ; k<COLONNEMAT ; k++){
						if(snts[k][0]==sn) find=true;			
					}
					if(!find){
						snts[sntsp][0]=sn;
						snts[sntsp][1]=ts;
						sntsp++;
						find=false;			
					}
					else find=false;
									
				}
			}
		}
	}
	
	/*for (int i=0; i<COLONNEMAT ; i++){
		std::cout << snts[i][0] << " ";
	}
	std::cout << std::endl;
	*/
}




//Questo metodo serve a trovare il sequence number pointer valido per una esecuzione dell'algoritmo di eliminazione gaussiana
int RtpFecModule::findsnp(){
	
	//azzero il vettore dei sn
	for(int i=0 ; i<COLONNEMAT ; i++){
		snpvett[i]=0;
	//copio i sn contenuti in snts nel vettore dei sn snpvett
	}
	for(int i=0 ; i<COLONNEMAT ; i++){
		snpvett[i]=snts[i][0];
	}
	
	//faccio un ordinamento del vettore dei sn
	sort(snpvett, snpvett+COLONNEMAT);
	
	int snpmin=0;	//puntatore al minimo sn
	int snpmax=0;	//puntatore al massimio sn
	int c=0;
	while(snpmin==0){
		snpmin=snpvett[c];
		c++;
	};	
	
	int minpointer=0; //nuovo puntatore al minimo sn
	//cerco la posizione del minimo sn all'interndo di snpvett e la salvo in minpointer
	for(int i=0 ; i<COLONNEMAT ; i++){
		if (snpvett[i]==0) continue;
		else{
			if(snpvett[i]<snpmin){
				snpmin=snpvett[i];
				minpointer=i;
			}	
		}	
	}	
	
	//cerco il valore del snpmax
	for(int i=0 ; i<COLONNEMAT ; i++){
		if(snpvett[i]>snpmax)snpmax=snpvett[i];
	}	
	
	//controllo che il valore di snpmax - numero di colonne della matrice sia maggiore del minsnp altrimenti cerco il nuovo snp che assova a tale 
	//condizione
	if(snpmax-COLONNEMAT>snpmin){
		//std::cout << "SONO QUI DENTRO" << std::endl;
		while(snpmax-COLONNEMAT>snpmin){
			snpmin=snpvett[minpointer+1];
		//	std::cout << "snpmin: " << snpmin << std::endl;
			minpointer++;
		}	
		//std::cout << "FINE SONO QUI DENTRO" << std::endl;	
	}	
	//std::cout << "snpmin: " << snpmin << std::endl;
	//std::cout << "snpmax: " << snpmax << std::endl;
	
	return snpmin;
}



//metodo che crea la matrice di coefficienti necessaria per l'eliminazione gaussiana
void RtpFecModule::creamatrice(){
	
	//azzero la matrice
	for(int i=0;i<RIGHE;i++){		//Ciclo da 0 fino a RIGHE
		for(int j=0;j<COLONNEMAT;j++){	//Ciclo da 0 fino a COLONNE
			mat[i][j]=0;		//Azzero gli elementi della matrice dei coefficienti
		}	
	}	
	
	int rigamat=0;
	for(int i=0 ; i<RIGHE ; i++){
		Packet* tmp = pckrecv[i];
		hdr_cmn* ch = hdr_cmn::access(tmp);
		if(ch->ptype()==PT_MCBR){
			hdr_cbr* cbrh = hdr_cbr::access(tmp);
			int sn=cbrh->sn;
			//std::cout << "sn: " << sn << std::endl;
			if(sn>=snp){
				for(int j=0 ; j<COLONNEMAT ; j++){
					if(snp+j==sn){ 
						mat[rigamat][j]=1;	
						rigamat++;					
					}//end if				
				}//end for			
			}//end if
			
		}//end if
		else{
			hdr_rtpfec* rtph = hdr_rtpfec::access(tmp);
			if(rtph->snb>=snp){
				int pos = rtph->snb-snp;
				for (int j=0 ; j<MASCHERA ; j++){
					mat[rigamat][pos+j]=rtph->mask[j];
				}			
				rigamat++;
			}
			
		}//end else


	}//end for

	/*std::cout << "MATRICE INIZIALE" << std::endl;
	for(int i=0 ; i<RIGHE ; i++){
		for(int j=0 ; j<COLONNEMAT ; j++){
			std::cout << mat[i][j];	
		}
		std::cout << std::endl;
	}
	std::cout << std::endl;
	*/

}



//codice per l'esecuzione dell'algoritmo di eliminazione gaussiana
void RtpFecModule::elgauss(){

	/*std::cout << "MATRICE ORIGINALE" << std::endl;
	for(int i = 0 ; i<RIGHE ; i++){
			for(int j=0 ; j<COLONNE ; j++){
				std::cout << mat[i][j] << " ";		
			}
			std::cout << std::endl;
	}
	std::cout << "MATRICE ORIGINALE" << std::endl;*/

	bool find = false;	//flag per cercare l'elemento di pivoting
	int row=0;		//Indicatore della riga iniziale della ricerca dell'elemento di pivot
	int col=0;		//Indicatore della colonna iniziale della ricerca dell'elemento di pivot
	int r,c;		//Servono per ciclare la matrice
	r=row;
	c=col;
	while(col!=COLONNEMAT){
		find=false;		//Ad ogni ciclo rimetto find=false	
		r=row;			//Ad ogni ciclo sett r con il valore di row
		
		//std::cout << "row= " << row << std::endl;
		//std::cout << "col= " << col << std::endl;

		while(!find && r<RIGHE){			//Ciclo finchè find è true e finchè r < righe		
			if(mat[r][col]==1) find = true;		//Se l'elemento di posizione [r][col] è 1 setto find a true
			else r++;				//incremento il contatore delle righe
		}

		if(find==false){ 				//Controllo se find è false
			row++;					//Incremento la riga del pivot
			col++;					//Incremento la colonna del pivot
			continue;
		}

		if(row!=r) inverti(row,r);			//Se la riga di pivoting è diversa da quella in cui ho trovato l'elemento di pivoting le scambio
	
	
		for(int r=0;r<RIGHE;r++){			//Ciclo le righe della matrice
			if(r==row) continue;			//Controllo se la riga in cui sono è uguale a quella di pivoting se si vado alla prossima riga
			if (mat[r][col]==1) xorarighe(row,r);	//Se nell [r][col] ho un 1 faccio lo xor tra la riga attuale e quella di pivoting
		}
		
		row++;						//Incremento la riga di pivoting
		col++;						//Incremento la colonna di pivoting
	}
	
	/*std::cout << "MATRICE FINALE" << std::endl;
	for(int i = 0 ; i<RIGHE ; i++){
			for(int j=0 ; j<COLONNEMAT ; j++){
				std::cout << mat[i][j] << " ";		
			}
			std::cout << std::endl;
	}
	std::cout << "MATRICE FINALE" << std::endl;
	*/


/*Segue ora il codice che decide quali elementi sono recuperabili. Come prima viene azzerato il vettore
recuperabili contenente i sn effettivamente recuperabili dall'algoritmo di eliminazione gaussiana. Una 
volta azzerato il vettore inizia un ciclo che scorre tutta la matrice riorganizzata dall'algoritmo
di eliminazione gaussiana. Viene controllata ogni riga e si tiene traccia degli uni presenti. Se in una 
riga è presente un solo 1 allora viene aggiunto all'interno del vettore recuperabili il valore del sn
corrispondente a quell'1 in particlar modo uguale al snp(che rappresenta il sn + piccolo recuperabile)
+ il valore i della colonna in cui era presente l'1. Alla fine del processo ovvero quando sono stati 
inseriti all'interno di recuperabili tutti i sn effettivamente recuperabili si procede ad un riordinamento
in ordine crescente del vettore recuperabili. Questo viene fatto per permettere successivamente di decidere
quali sn inviare al modulo superiore e quali no*/

	for(int i=0 ; i<RIGHE ; i++){			//Ciclo da 0 fino a RIGHE
		recuperabili[i]=0;			//Metto a 0 il vettore recuperabili
	}

	int cont=0;					//Tiene traccia di quanti 1 ci sono in una riga
	int d;						//Memorizza la colonna in cui è presente un 1
	int riga=0;					//Puntantore del vettore recuperabili
	for(int i=0 ; i<RIGHE ; i++){			//Ciclo da 0 a RIGHE
		for(int j=0 ; j<COLONNEMAT ; j++){	//Ciclo da 0 a COLONNE
			if(mat[i][j]==1){		//Controllo se il valore della posizione attuale della matrice è 1
				cont++;			//Incremento il contatore degli 1
				d=j;			//Salvo il valore della colonna
			}		
		}
		if(cont==1){				//Se il contatore degli 1 è 1
			recuperabili[riga]=snp+d;	//Aggiungo il sn recuperabile all'interno di recuperabili
			riga++;				//Incremento il puntatore di recuperabili
		}

	cont=0;						//Azzero il contatore degli 1
	}	
	sort(recuperabili, recuperabili+RIGHE);		//Riordino in ordine crescente il vettore recuperabili


	/*std::cout << "RECUPERABILI" << std::endl;
	for(int i=0 ; i<RIGHE ; i++){
		std::cout << recuperabili[i] << " ";
	}
	std::cout << std::endl;
	std::cout << "RECUPERABILI" << std::endl;
	*/
}

/*Questo metodo accetta in ingresso due interi che corrispondono agli indici delle righe di una matrice sulli
quali si vuole operare.Ciclando le colonne della matrice esegue lo xor delle due righe poizione per posizione*/

void RtpFecModule::xorarighe(int r1, int r2){
	//std::cout << "xoro le righe: " << r1 << " e " << r2 << std::endl << std::endl; 

	for(int i=0 ; i<COLONNEMAT ; i++){			//Ciclo le colonne della matrice
		mat[r2][i]=mat[r2][i]^mat[r1][i];	//Faccio lo xor tra le due righe
	}

	/*std::cout << "MATRICE CON RIGHE " << r1 << " E " << r2 << " XORATE" << std::endl;
	for(int i = 0 ; i<RIGHE ; i++){
			for(int j=0 ; j<COLONNE ; j++){
				std::cout << mat[i][j] << " ";		
			}
			std::cout << std::endl;
	}
	std::cout << std::endl;*/
	
}



/*Questo metodo accetta in ingresso due interi che corrispondono agli indici delle righe di una matrice sulle queli 
si vuole operare. Mediante l'ausilio di un vettore temporaneo effettua lo scambio tra due righe della matrice*/

void RtpFecModule::inverti(int r1,int r2){

	//std::cout << "scambio le righe: " << r1 << " e " << r2 << std::endl << std::endl;
	
	int temp[COLONNEMAT];			//Definisco un vettore temporaneo lungo COLONNE
	for (int i=0; i<COLONNEMAT ; i++){		//Ciclo da 0 a COLONNE
		temp[i]=mat[r1][i];		//Copio nel vettore temp la riga r1
	}
	for (int i=0; i<COLONNEMAT ; i++){		//Ciclo da 0 a COLONNE
		mat[r1][i]=mat[r2][i];		//Copio nella riga r1 della matrice la riga r2
	}
	for (int i=0; i<COLONNEMAT ; i++){		//Ciclo da 0 a COLONNE
		mat[r2][i]=temp[i];		//Copio nella riga r2 il vettore temp
	}

	/*std::cout << "MATRICE CON RIGHE " << r1 << " E " << r2 << " SCAMBIATE" << std::endl;
	for(int i = 0 ; i<RIGHE ; i++){
			for(int j=0 ; j<COLONNE ; j++){
				std::cout << mat[i][j] << " ";		
			}
			std::cout << std::endl;
	}
	std::cout << std::endl;*/
};


void RtpFecModule::spedisci(){
	int sn;						   //Definisco una variabile sn
	double ts;					   //Definisco una variabile ts
	for (int i=0 ; i<RIGHE ; i++){			   //Ciclo il vettore recuperabili
		if(recuperabili[i]!=0){			   //Controllo se il valore iesimo è diverso da 0
			sn=recuperabili[i];		   //Salvo in sn il valore nella posizione iesima del vettore recuperabili
			for(int j=0 ; j<COLONNEMAT ; j++){	   //Ciclo la matrice snts
				if(sn==snts[j][0]){	   //Controllo se trovo il valore salvato in sn dentro la matrice
					ts = snts[j][1];   //Setto ts con il valore snts[j][1] corrispondente al sn		
					crea(sn,ts);	   //Chiamo il metodo crea passandogli sn e ts
				}
			}		
		}
	}

}

int speditipunt=0;		//Setto un puntatore dei pacchetti spediti

/*Questo metodo compie due operazioni.La principale è quella di creare i pacchetti cbr a partire dai sn e di passarli
al modulo superiore. La seconda funzione è quella di controllare se il un pacchetto con il sn attuale era già stato inviato 
in modo da evitare la duplicazione dei pacchetti al livello applicazione*/

void RtpFecModule::crea(int x, double y){
	
	bool trovato=false;				//Setto un flag a false
	for(int i=0 ; i<65000 ; i++){			//Scorro il vettore di pacchetti spediti
		if(x==spediti[i]) trovato=true;		//Controllo se il sn attuale è già stato spedito
	}
	if(!trovato){					//Se il sn non è stato ancora spedito
		/*if(x>2){
			if(x!=spediti[speditipunt-1]+1) exit(0);
		}*/		
		//std::cout << "sto inviando: " << x << std::endl;
		spediti[speditipunt]=x;			//Aggiungo il nuovo sn al vettore dei pacchetti spediti
		speditipunt++;				//Incremento il puntatore dei pacchetti spediti
		trovato=false;				//Setto il flag a false
		//std::cout << "Sto spedendo sn: " << x << " ts: " << y << std::endl;
		double delay = 0;			//Definisco il delay di invio
		Packet* p = Packet::alloc();		//Alloco il pacchetto p
		hdr_cmn* ch = hdr_cmn::access(p);	//Carico il common header di p
  		//ch->uid() = uidcnt_++;			
  		ch->ptype() = PT_MCBR;			//Setto il tipo di pacchetto come PT_MCBR
  		ch->size() = pcktsize;		//Setto la dimensione del pacchetto
  		
  		hdr_ip* iph = hdr_ip::access(p);	//Carico l'header ip
  		iph->daddr() = destaddr;			//Setto l'indirizzo di destinazione
  		iph->dport() = destport;		//Setto la porta di destinazione

  		hdr_cbr* cbrh = HDR_CBR(p);		//Carico l'header cbr
  		cbrh->sn = x;				//Setto il sn
  		cbrh->ts = y;				//Setto il ts
  		ch->timestamp() = cbrh->ts;		//Associo al pacchetto intero il ts
	
		sendUp(p,delay);			//Spedisco il file al modulo superiore
		
	}
	else{						//Se il sn è stato trovato
		trovato=false;				//Risetto il flag a false
	}

	
	/*if(x>273){
		for(int i=0 ; i<280; i++){
		std::cout << spediti[i] << " ";		
		int a=spediti[i];
		int b=spediti[i+1];
		if(b==0) break;				
		if(b!=a+1) exit(0);
		
		//if(spediti[i++]!=spediti[i]+1) break;
		}
	}*/

}



