/*
 * Copyright (c) 2008 Regents of the SIGNET lab, University of Padova.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University of Padova (SIGNET lab) nor the 
 *    names of its contributors may be used to endorse or promote products 
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RTPFEC_MODULE_H
#define	RTPFEC_MODULE_H
#define RIGHE 60
#define COLONNE 80
#define MASCHERA 24
#define COLONNEMAT RIGHE*MASCHERA
//Carico le librerie necessarie
#include <module.h>
#include<queue>

extern packet_t PT_RTPFEC;


//Definisco ls struttura dell'header rtpfec per ora solo con il sn
typedef struct hdr_rtpfec {

  int snb; //sn base del pacchetto dell'header fec 
  int sn;  
  bool mask[MASCHERA];
  double ts[MASCHERA];

  static int offset_;
  inline static int& offset() { return offset_; }
  inline static struct hdr_rtpfec* access(const Packet* p) {
    return (struct hdr_rtpfec*)p->access(offset_);
  }
  
} hdr_rtpfec;



//Classe che estende Module e specifica i metodi del nuovo modulo rtpfec
class RtpFecModule : public Module
{
	public:
		RtpFecModule(); //Costruttore del modulo

		virtual ~RtpFecModule();//Decostruttore del modulo
	
		virtual void recv(Packet *p);//Metodo che riceve i pacchetti
		
		virtual int command(int argc, const char*const* argv);//Metodo per specificare i comandi tcl


	protected:
 		void sendpkt();//Metodo per inviare i pacchetti
	
	  	void makepkt0(Packet* p);//Metodo per creare i pacchetti con codifica tipo 0

		void makepkt1(Packet* p);//Metodo per creare i pacchetti con codifica tipo 1

		void makepkt2();//Metodo per creare i pacchetti con codifica tipo 2

		void spedisci();	//Metodo per l'invio dei pacchetti cbr 
	
		void crea(int x, double y);	//Metodo che crea i pacchetti cbr
		
		int minore(int x, int y);//Metodo che restituisce il minore tra due numeri

		int maggiore(int x, int y);

		int findsnp();

		void creasnts();

		int snpvett[COLONNEMAT];

		int codec;	//Variabile che setta il tipo di codec
	
		int pckp; //Puntatore al vettore dei pck ricevuti		
	
		int mat[RIGHE][COLONNEMAT];

		int spediti[65000];	//Vettore dei sn spediti
			
		int speditipunt;	//Puntatode del vettore dei sn spediti

		//int rec[RIGHE][COLONNE];

		double snts[COLONNEMAT][5];	//Matrice che contiene i sn e i relativi ts
	
		int recuperabili[RIGHE];	//Vettore degli elementi recuperabili

		Packet* pckrecv[RIGHE];		//Vettore che contiene i pacchetti ricevuti
		

		void elgauss(); //richiede la matrice su cui operare
		
		void inverti(int r1,int r2);	//Metodo che inverte due righe
	
		void xorarighe(int r1, int r2);		//Metodo che xora due righe
	
		void creamatrice();		//Metodo che crea una matrice dei coefficienti		

		std::queue<Packet*> Q; // Coda per contenere i pacchetti da modificare

		int cont; // Un contatore
};


#endif /* RTPFEC_MODULE_H */
